<!-- Do not edit this file!                                 -->
<!-- This file is autogenerated with                        -->
<!--   tools/autotag/tag_script.py                          -->
<!-- Disable lints since this is an auto-generated file.    -->
<!-- markdownlint-disable blanks-around-headers             -->
<!-- markdownlint-disable no-duplicate-header               -->
<!-- markdownlint-disable no-blanks-blockquote              -->
<!-- markdownlint-disable ul-indent                         -->
<!-- markdownlint-disable no-trailing-spaces                -->
<!-- markdownlint-disable reference-links-images            -->
<!-- markdownlint-disable no-missing-space-atx              -->
<!-- spellcheck-disable                                     -->
{%- for version, release in releases %}
# ROCm {{version}} release notes

{%- set highlights = "./highlights/" ~ version ~ ".md" %}
{%- include highlights ignore missing -%}
{{- "\n\n" -}}
{%- set support = "./support/" ~ version ~ ".md" %}
{%- include support ignore missing -%}
{{- "\n\n" -}}

## ROCm components

<div class="pst-scrollable-table-container">
    <table id="rocm-rn-components" class="table">
        <thead>
            <tr>
                <th>Category</th>
                <th>Group</th>
                <th>Name</th>
                <th>Version</th>
                <th></th>
            </tr>
        </thead>
        <colgroup>
            <col span="1">
            <col span="1">
        </colgroup>
        <tbody>
{%- for lib_name in release.libraries %}
{%- set lib = release.libraries[lib_name] %}
{%- if rocm_ver_by_lib_ver[lib_name][lib.lib_version] == version and (prev_lib_ver[lib_name][lib.lib_version] | default([]) | length > 0) and lib.lib_version %}
            <tr>
                <th>{{ lib.category }}</th>
                <th>{{ lib.group }}</th>
                <td><a href="{{ lib.documentation_page }}">{{ lib_name }}</a></td>
                <td>{{ prev_lib_ver[lib_name][lib.lib_version] }}&nbsp;&Rightarrow;&nbsp;<a href="{{ lib.release_url }}">{{ lib.lib_version }}</a></td>
                <td><a href="{{ lib.repository_url }}"><i class="fab fa-github fa-lg"></i></a></td>
            </tr>
{%- elif lib.lib_version %}
            <tr>
                <th>{{ lib.category }}</th>
                <th>{{ lib.group }}</th>
                <td><a href="{{ lib.documentation_page }}">{{ lib_name }}</a></td>
                <td><a href="{{ lib.release_url }}">{{ lib.lib_version }}</a></td>
                <td><a href="{{ lib.repository_url }}"><i class="fab fa-github fa-lg"></i></a></td>
            </tr>
{%- endif %}
{%- endfor %}{# lib_name in release.libraries #}
        </tbody>
    </table>
</div>

{%- for lib_name, lib in release.libraries | dictsort %}
{%- if rocm_ver_by_lib_ver[lib_name][lib.lib_version] == version and lib.lib_version%}

### **{{lib_name}}** ({{ lib.lib_version }})

{%- for type, change in lib.data.changes.items() %}

#### {{type|title}}

{{change|trim|e}}

{%- endfor %}{# change in lib.data.changes #}
{%- endif -%}
{%- endfor %}{# lib in release.libraries #}
{{- "\n\n" -}}
{%- set extra_components = "./extra_components/" ~ version ~ ".md" %}
{%- include extra_components ignore missing -%}
{{- "\n\n" -}}
{%- set known_issues = "./known_issues/" ~ version ~ ".md" %}
{%- include known_issues ignore missing -%}
{{- "\n\n" -}}
{%- set resolved_issues = "./resolved_issues/" ~ version ~ ".md" %}
{%- include resolved_issues ignore missing -%}
{{- "\n\n" -}}
{%- set upcoming_changes = "./upcoming_changes/" ~ version ~ ".md" %}
{%- include upcoming_changes ignore missing -%}

{%- endfor %}{# release in releases #}
{# EOF #}
